<?php
/**
 * Copyright © 2015 Magenest. All rights reserved.
 * See COPYING.txt for license details.
 *
 * Magenest_Salesforce extension
 * NOTICE OF LICENSE
 *
 * @category Magenest
 * @package  Magenest_Salesforce
 * @author   ThaoPV
 */
namespace Magenest\Salesforce\Controller\Adminhtml\Report;

use Magenest\Salesforce\Controller\Adminhtml\Report as ReportController;
use Magenest\Salesforce\Model\ReportFactory as ReportFactory;
use Magenest\Salesforce\Model\ResourceModel\Report\CollectionFactory as ReportCollectionFactory;
use Magento\Backend\App\Action\Context;
use Magento\Backend\Model\View\Result\ForwardFactory;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\View\LayoutFactory;
use Magento\Framework\View\Result\PageFactory;
use Magento\Ui\Component\MassAction\Filter;

/**
 * Class Index
 *
 * @package Magenest\Salesforce\Controller\Adminhtml\Report
 */
class MassDelete extends ReportController
{
    /**
     * Mass Action Filter
     *
     * @var Filter
     */
    protected $_filter;

    /**
     * Report Collection
     *
     * @var ReportCollectionFactory
     */
    protected $_collectionFactory;

    /**
     * Report Model
     *
     * @var ReportFactory $_reportFactory
     */
    protected $_reportFactory;

    /**
     * @param Context                 $context
     * @param ReportFactory           $reportFactory
     * @param LayoutFactory           $layoutFactory
     * @param PageFactory             $resultPageFactory
     * @param Filter                  $filter
     * @param ForwardFactory          $resultForwardFactory
     * @param ReportCollectionFactory $collectionFactory
     */
    public function __construct(
        Context $context,
        ReportFactory $reportFactory,
        LayoutFactory $layoutFactory,
        PageFactory $resultPageFactory,
        Filter $filter,
        ForwardFactory $resultForwardFactory,
        ReportCollectionFactory $collectionFactory
    ) {
        $this->_reportFactory = $reportFactory;
        $this->_filter            = $filter;
        $this->_collectionFactory = $collectionFactory;
        parent::__construct($context, $reportFactory, $layoutFactory, $resultPageFactory, $resultForwardFactory);
    }

    /**
     * @return \Magento\Backend\Model\View\Result\Redirect|\Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function execute()
    {
        $collectionFilter = $this->_filter->getCollection($this->_collectionFactory->create());
        /** @var \Magenest\Salesforce\Model\Report $reportModel */
        $reportModel = $this->_reportFactory->create();
        $collectionSize = $collectionFilter->getSize();
        $collectionArr = [];
        $maxRecord = 5000;
        $count = 0;
        $lastItemId = $collectionFilter->getLastItem()->getId();
        foreach ($collectionFilter as $item) {
            $count++;
            /** @var \Magenest\Salesforce\Model\Map $item */
            $collectionArr[] = $item->getId();
            if ($count >= $maxRecord || $item->getId() == $lastItemId) {
                $reportModel->deleteMultiReports($collectionArr);
                $collectionArr = [];
                $count = 0;
            }
        }

        $this->messageManager->addSuccessMessage(__('A total of %1 record(s) have been deleted.', $collectionSize));
        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
        return $resultRedirect->setPath('*/*/');
    }

    /**
     * @return bool
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('Magenest_Salesforce::salesforce_report');
    }
}
