<?php
namespace Magenest\Salesforce\Controller\Adminhtml\Sync;

use Magenest\Salesforce\Model\Sync;
use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;

/**
 * Class Product
 * @package Magenest\Salesforce\Controller\Adminhtml\Sync
 */
class Product extends Action
{
    /**
     * @var Sync\Product
     */
    protected $_product;

    /**
     * Customer constructor.
     * @param Context $context
     * @param Sync\Product $product
     */
    public function __construct(
        Context $context,
        Sync\Product $product
    ) {
        $this->_product = $product;
        parent::__construct($context);
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\Result\Redirect|\Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $productId = $this->getRequest()->getParam('id');
        try {
            if ($productId) {
                $this->_product->sync($productId);
                $this->messageManager->addSuccessMessage(
                    __('Product sync process complete, check out Reports for result.')
                );
            } else {
                $this->messageManager->addNoticeMessage(
                    __('No product has been selected')
                );
            }
        } catch (\Throwable $e) {
            $this->messageManager->addErrorMessage(
                __('Something happen during syncing process. Detail: ' . $e->getMessage() . ". Trace: " . $e->getTraceAsString())
            );
        }
        $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
        $resultRedirect->setPath('catalog/product/edit', ['id' => $productId]);
        return $resultRedirect;
    }

    /**
     * @return bool
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('Magenest_Salesforce::config_salesforce');
    }
}
