<?php
namespace Magenest\Salesforce\Model\Sync;

use Magenest\Salesforce\Helper\Data as Helper;
use Magenest\Salesforce\Logger\Logger;
use Magenest\Salesforce\Model\Data;
use Magenest\Salesforce\Model\ReportFactory;
use Magenest\Salesforce\Model\RequestLogFactory;
use Magenest\Salesforce\Model\ResourceModel\Queue as QueueResource;
use Magenest\Salesforce\Model\ResourceModel\Queue\CollectionFactory as QueueCollectionFactory;
use Magenest\Salesforce\Model\ResourceModel\Report\CollectionFactory as ReportCollectionFactory;
use Magento\Config\Model\ResourceModel\Config as ResourceModelConfig;
use Magento\Customer\Model\CustomerFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\HTTP\ClientFactory;
use Magento\Framework\Model\ResourceModel\Iterator;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\Serialize\Serializer\Serialize;
use Magento\Newsletter\Model\ResourceModel\Subscriber\Collection;
use Magento\Newsletter\Model\ResourceModel\Subscriber\CollectionFactory;

class Subscriber extends Contact
{
    /** @var CollectionFactory */
    private $subscriberCollection;

    /**
     * @param CustomerFactory $customerFactory
     * @param Data $data
     * @param Job $job
     * @param DataGetter $dataGetter
     * @param Helper $helper
     * @param Lead $lead
     * @param Account $account
     * @param ReportCollectionFactory $reportCollectionFactory
     * @param Helper $helperData
     * @param Json $json
     * @param Serialize $serialize
     * @param ScopeConfigInterface $scopeConfig
     * @param ResourceModelConfig $resourceConfig
     * @param ReportFactory $reportFactory
     * @param QueueCollectionFactory $queueCollectionFactory
     * @param QueueResource $queueResource
     * @param RequestLogFactory $requestLogFactory
     * @param ClientFactory $client
     * @param Iterator $iterator
     * @param Logger $logger
     * @param CollectionFactory $subscriberCollection
     */
    public function __construct(
        CustomerFactory $customerFactory,
        Data $data,
        Job $job,
        DataGetter $dataGetter,
        Helper $helper,
        Lead $lead,
        Account $account,
        ReportCollectionFactory $reportCollectionFactory,
        Helper $helperData,
        Json $json,
        Serialize $serialize,
        ScopeConfigInterface $scopeConfig,
        ResourceModelConfig $resourceConfig,
        ReportFactory $reportFactory,
        QueueCollectionFactory $queueCollectionFactory,
        QueueResource $queueResource,
        RequestLogFactory $requestLogFactory,
        ClientFactory $client,
        Iterator $iterator,
        Logger $logger,
        CollectionFactory $subscriberCollection
    ) {
        parent::__construct(
            $customerFactory,
            $data,
            $job,
            $dataGetter,
            $helper,
            $lead,
            $account,
            $reportCollectionFactory,
            $helperData,
            $json,
            $serialize,
            $scopeConfig,
            $resourceConfig,
            $reportFactory,
            $queueCollectionFactory,
            $queueResource,
            $requestLogFactory,
            $client,
            $iterator,
            $logger
        );
        $this->_type = 'Subscriber';
        $this->subscriberCollection = $subscriberCollection;
    }

    /**
     * @param \Magento\Newsletter\Model\Subscriber $subscriber
     * @return bool
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function syncOne(\Magento\Newsletter\Model\Subscriber $subscriber)
    {
        try {
            if ($subscriber->getCustomerId() == 0) {
                if ($subscriber->getStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED) {
                    $this->lead->syncByEmail($subscriber->getEmail());
                } else {
                    $this->lead->delete($subscriber->getEmail());
                }
            } elseif ($field = $this->helper->getSubscriberAccountField()) {
                $this->account->sync(
                    $subscriber->getCustomerId(),
                    ['field' => $field, 'status' => $subscriber->getStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED]
                );
            }
        } catch (\Throwable $e) {
            $this->saveReport(
                null,
                'update',
                'Subscriber',
                2,
                $e->getMessage(),
                $subscriber->getCustomerId(),
                $e->getTrace()
            );

            return false;
        }

        return true;
    }

    /**
     * @return void
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function syncAllQueue()
    {
        $ids = $this->_queueResource->fetchEntityIdByType($this->_type);
        if (!empty($ids)) {
            $collection = $this->subscriberCollection->create()
                ->addFieldToFilter('subscriber_id', ['in' => $ids])
                ->showCustomerInfo();
            /** @var \Magento\Newsletter\Model\Subscriber $item */
            foreach ($collection->getItems() as $item) {
                $this->syncOne($item);
            }

            $this->_queueResource->deleteMultiQueues(array_keys($ids));
        }
    }
}
