<?php
namespace Magenest\Salesforce\Observer;

use Magenest\Salesforce\Helper\Data;
use Magenest\Salesforce\Logger\Logger;
use Magenest\Salesforce\Model\QueueFactory;
use Magenest\Salesforce\Model\ResourceModel\Queue;
use Magenest\Salesforce\Model\ResourceModel\Queue\CollectionFactory;
use Magenest\Salesforce\Model\ResourceModel\Report;
use Magenest\Salesforce\Model\Sync\Account;
use Magenest\Salesforce\Model\Sync\Campaign;
use Magenest\Salesforce\Model\Sync\Contact;
use Magenest\Salesforce\Model\Sync\Lead;
use Magenest\Salesforce\Model\Sync\Opportunity;
use Magenest\Salesforce\Model\Sync\Order;
use Magenest\Salesforce\Model\Sync\Product;
use Magenest\Salesforce\Model\Sync\Subscriber;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\Event\ObserverInterface;
use Magento\Sales\Api\OrderRepositoryInterface;

/**
 * Class SyncObserver
 * @package Magenest\Salesforce\Observer
 */
abstract class SyncObserver implements ObserverInterface
{
    protected $type;

    /** @var OrderRepositoryInterface */
    protected $orderRepository;

    /** @var ProductRepositoryInterface */
    protected $productRepository;

    /** @var QueueFactory */
    protected $queueFactory;

    /** @var Queue */
    protected $queueResource;

    /** @var CollectionFactory */
    protected $queueColFactory;

    /** @var Data */
    protected $helper;

    /** @var Logger */
    protected $logger;

    /** @var Campaign */
    protected $_campaign;

    /** @var Lead */
    protected $_lead;

    /** @var Contact */
    protected $_contact;

    /** @var Account */
    protected $_account;

    /** @var Order */
    protected $_order;

    /**  @var Opportunity */
    protected $_opportunity;

    /** @var Product */
    protected $_product;

    /** @var Report */
    protected $_reportResourceModel;

    /** @var Subscriber */
    protected $subscriber;

    /**
     * SyncObserver constructor.
     * @param OrderRepositoryInterface $orderRepository
     * @param ProductRepositoryInterface $productRepository
     * @param QueueFactory $queueFactory
     * @param Queue $queueResource
     * @param CollectionFactory $queueColFactory
     * @param Data $helper
     * @param Logger $logger
     * @param Campaign $campaign
     * @param Lead $lead
     * @param Contact $contact
     * @param Account $account
     * @param Order $order
     * @param Opportunity $opportunity
     * @param Product $product
     * @param Subscriber $subscriber
     * @param Report $reportResourceModel
     */
    public function __construct(
        OrderRepositoryInterface   $orderRepository,
        ProductRepositoryInterface $productRepository,
        QueueFactory               $queueFactory,
        Queue                      $queueResource,
        CollectionFactory          $queueColFactory,
        Data                       $helper,
        Logger                     $logger,
        Campaign                   $campaign,
        Lead                       $lead,
        Contact                    $contact,
        Account                    $account,
        Order                      $order,
        Opportunity                $opportunity,
        Product                    $product,
        Subscriber                 $subscriber,
        Report                     $reportResourceModel
    ) {
        $this->orderRepository      = $orderRepository;
        $this->productRepository    = $productRepository;
        $this->helper               = $helper;
        $this->logger               = $logger;
        $this->queueFactory         = $queueFactory;
        $this->queueResource        = $queueResource;
        $this->queueColFactory      = $queueColFactory;
        $this->_campaign            = $campaign;
        $this->_lead                = $lead;
        $this->_contact             = $contact;
        $this->_account             = $account;
        $this->_order               = $order;
        $this->_opportunity         = $opportunity;
        $this->_product             = $product;
        $this->subscriber           = $subscriber;
        $this->_reportResourceModel = $reportResourceModel;
    }

    /**
     * @param string $type
     * @return bool
     */
    public function getEnabledSyncConfig($type = '')
    {
        if ($type) {
            return $this->helper->getEnabledSyncConfig($type);
        }
        return $this->helper->getEnabledSyncConfig($this->type);
    }

    /**
     * @param string $type
     * @return int
     */
    public function getSyncConfigMode($type = '')
    {
        if ($type) {
            return $this->helper->getSyncConfigMode($type);
        }
        return $this->helper->getSyncConfigMode($this->type);
    }

    /**
     * @param $type
     * @return int
     */
    public function getSyncConfigTime($type = '')
    {
        if ($type) {
            return $this->helper->getSyncConfigTime($type);
        }
        return $this->helper->getSyncConfigTime($this->type);
    }

    /**
     * @param string $type
     * @param int $entityId
     * @throws \Exception
     */
    public function addToQueue($type, $entityId)
    {
        /** add to queue mode */
        $queue = $this->queueColFactory->create()
            ->addFieldToFilter('type', $type)
            ->addFieldToFilter('entity_id', $entityId)
            ->getFirstItem();
        if ($queue->getId()) {
            $queue->setEnqueueTime(time());
        } else {
            $queue = $this->queueFactory->create();
            $data  = [
                'type'         => $type,
                'entity_id'    => $entityId,
                'enqueue_time' => time(),
                'priority'     => 1,
            ];
            $queue->setData($data);
        }
        $this->queueResource->save($queue);
        return;
    }
}
